package com.zanxiang.game.data.serve.service.impl;

import com.google.common.base.CaseFormat;
import com.zanxiang.game.data.serve.pojo.dto.GameServerDayDTO;
import com.zanxiang.game.data.serve.pojo.dto.GameServerDayTotalDTO;
import com.zanxiang.game.data.serve.pojo.entity.AdsGameServerDay;
import com.zanxiang.game.data.serve.pojo.enums.OrderByEnum;
import com.zanxiang.game.data.serve.pojo.vo.GameServerDayTotalVO;
import com.zanxiang.game.data.serve.pojo.vo.GameServerDayVO;
import com.zanxiang.game.data.serve.pojo.vo.GameServerTrendVO;
import com.zanxiang.game.data.serve.service.IGameServerService;
import com.zanxiang.game.data.serve.utils.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.nutz.dao.Cnd;
import org.nutz.dao.Dao;
import org.nutz.dao.Sqls;
import org.nutz.dao.pager.Pager;
import org.nutz.dao.sql.Criteria;
import org.nutz.dao.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.util.function.Tuple2;
import reactor.util.function.Tuples;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.time.LocalDate;
import java.util.*;

/**
 * @author tianhua
 * @time 2023/8/14
 * @Description 游戏区服接口逻辑处理
 **/
@Slf4j
@Service
public class GameServerServiceImpl implements IGameServerService {

    //存储映射的List
    private static final List<Tuple2<Field,Field>> dayNFieldMapList;
    //private static final List<Tuple2<Field,Field>> dayNTotalFieldMapList;

    static {
        //解析游戏区服的dayN映射
        Map<String, Field> fieldMap = new HashMap<>();
        List<Field> dayNFieldList = new ArrayList<>();
        Class<?> tempClazz = GameServerDayVO.class;
        while (tempClazz != null) {
            //得到所有的field
            Field[] fields = tempClazz.getDeclaredFields();
            for (Field field : fields) {
                if (Modifier.isFinal(field.getModifiers()) || Modifier.isStatic(field.getModifiers())) {
                    continue;
                }
                //map里面存储了所有的field
                fieldMap.put(field.getName(), field);
                if (field.getType() == GameServerTrendVO.class) {
                    //list里面只存储了 Trend 相关的field
                    dayNFieldList.add(field);
                }
            }
            tempClazz = tempClazz.getSuperclass();
        }
        if (dayNFieldList.isEmpty()) {
            dayNFieldMapList = Collections.emptyList();
        } else {
            dayNFieldMapList = new ArrayList<>(dayNFieldList.size());
            for (Field dayNField : dayNFieldList) {
                dayNField.setAccessible(true);
                Field sourceField = fieldMap.get(dayNField.getName().replace("Trend", ""));
                sourceField.setAccessible(true);
                if (sourceField != null) {
                    dayNFieldMapList.add(Tuples.of(sourceField, dayNField));
                }
            }
        }
    }


    @Autowired
    private Dao dao;

    /**
     * 游戏区服数据
     *
     * @param dto 前端传递的查询参数
     * @return page
     */
    @Override
    public Page<GameServerDayVO> getGameServerDataDay(GameServerDayDTO dto) {

        //不传递时间，默认查询当天数据
        if (dto.getBeginDate() == null || dto.getEndDate() == null) {
            dto.setBeginDate(LocalDate.now());
            dto.setEndDate(LocalDate.now());
        }
        //默认排序字段 开服时间 降序
        if (StringUtils.isBlank(dto.getSortFiled())) {
            dto.setSortFiled("dt");
        }
        if (StringUtils.isBlank(dto.getSortType())) {
            dto.setSortType(OrderByEnum.DESC.getOrderType());
        }
        //创建查询条件、
        Criteria cri = Cnd.cri();
        if (dto.getGameId() != null) {
            //拼接游戏ID查询条件
            cri.where().andEquals("game_id", dto.getGameId());
        }
        if (dto.getServerId() != null) {
            //拼接区服ID查询条件
            cri.where().andEquals("server_id", dto.getServerId());
        }
        if (StringUtils.isNotBlank(dto.getServerName())) {
            //拼接区服名称查询条件
            cri.where().andLike("server_name", dto.getServerName());
        }
        if (dto.getBeginDate() != null && dto.getEndDate() != null) {
            //拼接开服时间查询条件
            cri.where().andBetween("dt", dto.getBeginDate(), dto.getEndDate());
        }
        //添加排序条件
        //拼接排序条件
        cri.getOrderBy().orderBy(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, dto.getSortFiled()), dto.getSortType());
        //创建sql语句查询数据
        Sql sql = Sqls.create(gameServerDaySql() + cri);
        //设置自定义回传对象
        sql.setCallback(Sqls.callback.entities());
        sql.setEntity(dao.getEntity(GameServerDayVO.class));
        //设置分页对象
        Pager pager = dao.createPager(dto.getPageNum(), dto.getPageSize());
        sql.setPager(pager);
        //执行sql
        dao.execute(sql);
        //设置总记录数
        pager.setRecordCount(dao.count(AdsGameServerDay.class, cri));
        //返回结果
        return new Page<>(sql.getList(GameServerDayVO.class), pager);
    }

    /**
     * 游戏区服数据总计
     *
     * @param dto 前端传递的查询参数
     * @return vo
     */
    @Override
    public GameServerDayTotalVO getGameServerDataDayTotal(GameServerDayTotalDTO dto) {
        return null;
    }

    /**
     * 游戏区服数据SQL
     * @return String
     */
    private String gameServerDaySql() {
        return """
                SELECT
                    dt,
                    source_system,
                    server_id,
                    server_name,
                    game_id,
                    game_name,
                    classify,
                    out_total_num,
                    out_total_amount_num,
                    out_total_amount,
                    out_total_rate,
                    total_role_num,
                    total_reg_num,
                    total_amount_num,
                    total_amount,
                    da1,
                    da2,
                    da3,
                    da4,
                    da5,
                    da6,
                    da7,
                    da8,
                    da9,
                    da10,
                    da11,
                    da12,
                    da13,
                    da14,
                    da15,
                    da16,
                    da17,
                    da18,
                    da19,
                    da20,
                    da21,
                    da22,
                    da23,
                    da24,
                    da25,
                    da26,
                    da27,
                    da28,
                    da29,
                    da30,
                    m2,
                    m3,
                    m4,
                    m5,
                    m6,
                    m7,
                    m8,
                    m9,
                    m10,
                    m11,
                    y1,
                    total
                FROM
                    game_ads.ads_game_server_day
                """;
    }


}
